<?php

# handle better utf-8 and unicode encoding
if(function_exists('mb_internal_encoding')) { mb_internal_encoding('UTF-8'); }

# Consider fallback to PHP Minify [2016.08.01] from https://github.com/matthiasmullie/minify (must be defined on the outer scope)
require_once ($plugindir . 'libs/matthiasmullie/minify/src/Minify.php');
require_once ($plugindir . 'libs/matthiasmullie/minify/src/CSS.php');
require_once ($plugindir . 'libs/matthiasmullie/minify/src/JS.php');
require_once ($plugindir . 'libs/matthiasmullie/minify/src/Exception.php');
require_once ($plugindir . 'libs/matthiasmullie/path-converter/src/Converter.php');
use MatthiasMullie\Minify;

# use HTML minification
require_once ($plugindir . 'libs/mrclay/HTML.php');


# functions, get hurl info
function fastvelocity_min_get_hurl($src, $wp_domain, $wp_home) {
	
# preserve empty source handles
$hurl = trim($src); if(empty($hurl)) { return $hurl; }      

# some fixes
$hurl = str_ireplace(array('&#038;', '&amp;'), '&', $hurl);

# get current protocol scheme with cloudflare support
if (isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == 'on' || $_SERVER['HTTPS'] == 1) || isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https') { $protocol = 'https://'; } else { $protocol = 'http://'; }

#make sure wp_home doesn't have a forward slash
$wp_home = rtrim($wp_home, '/');

# apply some filters
if (substr($hurl, 0, 2) === "//") { $hurl = 'http://'.ltrim($hurl, "/"); }  # protocol only
if (substr($hurl, 0, 4) === "http" && stripos($hurl, $wp_domain) === false) { return $hurl; } # return if external domain
if (substr($hurl, 0, 4) !== "http" && stripos($hurl, $wp_domain) !== false) { $hurl = $wp_home.'/'.ltrim($hurl, "/"); } # protocol + home

# prevent double forward slashes in the middle
$hurl = str_ireplace('###', '://', str_ireplace('//', '/', str_ireplace('://', '###', $hurl)));

# consider different wp-content directory
$proceed = 0; if(!empty($wp_home)) { 
	$alt_wp_content = basename($wp_home); 
	if(substr($hurl, 0, strlen($alt_wp_content)) === $alt_wp_content) { $proceed = 1; } 
}

# protocol + home for relative paths
if (substr($hurl, 0, 12) === "/wp-includes" || substr($hurl, 0, 9) === "/wp-admin" || substr($hurl, 0, 11) === "/wp-content" || $proceed == 1) { 
$hurl = $wp_home.'/'.ltrim($hurl, "/"); }

# make sure there is a protocol prefix as required
$hurl = $protocol.str_ireplace(array('http://', 'https://'), '', $hurl); # enforce protocol

# no query strings
if (stripos($hurl, '?v') !== false) { $hurl = stristr($hurl, '?v', true); } # no query strings

return $hurl;	
}


# functions, minify html
function fastvelocity_min_minify_html($html) {
return trim(fastvelocity_min_Minify_HTML::minify($html));
}


# case-insensitive in_array() wrapper
function fastvelocity_min_in_arrayi($hurl, $ignore){
	$hurl = str_ireplace(array('http://', 'https://'), '//', $hurl); # better compatibility
	$hurl = strtok(urldecode(rawurldecode($hurl)), '?'); # no query string, decode entities
	
	if (!empty($hurl)) { 
		foreach ($ignore as $i) {
		$i = str_ireplace(array('http://', 'https://'), '//', $i); # better compatibility
		$i = strtok(urldecode(rawurldecode($i)), '?'); # no query string, decode entities
		$i = trim(trim($i, '/'), '*'); # wildcard removal
		if (stripos($hurl, $i) !== false) { return true; } 
		} 
	}
	return false;
}


# minify css string with PHP Minify
function fastvelocity_min_minify_css_string($css) {
$minifier = new Minify\CSS($css);
$minifier->setMaxImportSize(15); # [css only] embed assets up to 15 Kb (default 5Kb) - processes gif, png, jpg, jpeg, svg & woff
$min = $minifier->minify();
$min = str_ireplace(array('http://', 'https://'), '//', $min); # better compatibility
if($min !== false) { return $min; }
return $css;
}


# find if we are running windows
function fvm_server_is_windows() {
	if(defined('PHP_OS_FAMILY') && strtolower(PHP_OS_FAMILY) == 'windows') { return true; } # PHP 7.2.0+
	if (stripos(php_uname('s'), 'Windows') !== false) { return true; }# fallback
	return false;
}



# minify js on demand (one file at one time, for compatibility)
function fastvelocity_min_get_js($url, $js, $disable_js_minification) {

# exclude minification on already minified files + jquery (because minification might break those)
$excl = array('jquery.js', '.min.js', '-min.js'); 
foreach($excl as $e) { if (stripos(basename($url), $e) !== false) { $disable_js_minification = true; break; } }	

# remove BOM
$js = fastvelocity_min_remove_utf8_bom($js); 

# minify JS
if(!$disable_js_minification) { $js = fastvelocity_min_minify_js_string($js); }

# better compatibility urls
$js = str_ireplace(array('http://', 'https://'), '//', $js); 
$js = $js."\r\n;"; # needed when merging js files

# return html
return $js;
}


# minify JS string with PHP Minify or YUI Compressors
function fastvelocity_min_minify_js_string($js) {
global $tmpdir, $plugindir;

# check for exec + a supported java version
$use_yui = get_option('fastvelocity_min_use_yui');
if($use_yui == true && function_exists('exec') && exec('command -v java >/dev/null && echo "yes" || echo "no"') == 'yes') {

	# create temp files
	$tmpname = hash('adler32', $js);
	$tmpin = $tmpdir.'/'.$tmpname.'.js';
	file_put_contents($tmpin, $js);
	$tmpout = $tmpdir.'/'.$tmpname.'.min.js';
	
	# define jar path and command
	$cmd = 'java -jar '.$plugindir.'libs/jar/yuicompressor-2.4.8.jar'.' --preserve-semi --nomunge '.$tmpin.' -o '.$tmpout;
		
	# run local compiler
	exec($cmd . ' 2>&1', $output);
	if(count($output) == 0 && file_exists($tmpout)) {
		$min = file_get_contents($tmpout);
		if($min !== false) { return $min; }
		exit();
	}		
}

# PHP Minify [2016.08.01] from https://github.com/matthiasmullie/minify
$minifier = new Minify\JS($js);
$min = $minifier->minify();
if($min !== false && (strlen(trim($js)) == strlen(trim($min)) || strlen(trim($min)) > 0)) { return $min; }

# if we are here, something went  wrong and minification didn't work
$js = "\n/*! Fast Velocity Minify: Minification of the following section has failed, so it has been merged instead. */\n".$js;
return $js;
}


# functions to minify HTML
function fastvelocity_min_html_compression_finish($html) { return fastvelocity_min_minify_html($html); }
function fastvelocity_min_html_compression_start() { 
ob_start('fastvelocity_min_html_compression_finish'); 
}


# remove all cache files
function rrmdir($dir) { 
	if(is_dir(rtrim($dir, '/'))) { 
		if ($handle = opendir($dir.'/')) { 
			while (false !== ($file = readdir($handle))) { 
			@unlink($dir.'/'.$file); 
			} 
		closedir($handle); } 
	} 
}


# minify css on demand (one file at one time, for compatibility)
function fastvelocity_min_get_css($url, $css, $disable_css_minification) {

# remove BOM
$css = fastvelocity_min_remove_utf8_bom($css); 

# fix url paths
if(!empty($url)) { $css = preg_replace("/url\(\s*['\"]?(?!data:)(?!http)(?![\/'\"])(.+?)['\"]?\s*\)/ui", "url(".dirname($url)."/$1)", $css); } 

# better compatibility urls
$css = str_ireplace(array('http://', 'https://'), '//', $css); 

# fix bootstrap 4 svg images https://www.w3.org/TR/SVG/intro.html#NamespaceAndDTDIdentifiers
$css = str_ireplace('//www.w3.org/2000/svg', 'http://www.w3.org/2000/svg', $css); 

# remove query strings from fonts (for better seo, but add a small cache buster based on most recent updates)
$ctime = get_option('fvm-last-cache-update', '0'); # last update or zero
$css = preg_replace('/(.eot|.woff2|.woff|.ttf)+[?+](.+?)(\)|\'|\")/ui', "$1"."#".$ctime."$3", $css); # fonts cache buster

# minify CSS
if(!$disable_css_minification) { $css = fastvelocity_min_minify_css_string($css); }

# return html
return $css;
}



# get remote urls with curl
function fvm_file_get_contents_curl($url, $uagent=NULL) {
    $ch = curl_init();
	if(isset($uagent) && !empty($uagent)) { curl_setopt($ch,CURLOPT_USERAGENT, $uagent); }
    curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT , 10); 
	curl_setopt($ch, CURLOPT_TIMEOUT, 15);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    $data = curl_exec($ch);
    curl_close($ch);
    return $data;
}



# download and cache css and js files
function fvm_download_and_cache($hurl, $tkey, $inline=null, $disable_minification=false, $type=null){
global $cachedir, $cachedirurl, $wp_domain, $wp_home, $wp_home_path;

# filters and defaults
$uagent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/8.0.7 Safari/600.7.12';
$printurl = str_ireplace(array(site_url(), home_url(), 'http:', 'https:'), '', $hurl);

# try to open locally first, but skip if we are on windows
if(fvm_server_is_windows() === false) {
if (stripos($hurl, $wp_domain) !== false) { 
	# default
	$f = str_ireplace(rtrim($wp_home, '/'), rtrim($wp_home_path, '/'), $hurl);
	if (file_exists($f)) { 
		if($type == 'js') { $code = fastvelocity_min_get_js($hurl, file_get_contents($f), $disable_minification); } 
		else { $code = fastvelocity_min_get_css($hurl, file_get_contents($f).$inline, $disable_minification); }
		set_transient($tkey, $code, 7 * DAY_IN_SECONDS );
		fvm_update_transient_keys($tkey); # keep track
		$log = " - FILE &nbsp; - $printurl --- Debug: File was opened from $f ---\n";
		return array('log'=>$log, 'code'=>$code);
	}
	
	# failover when home_url != site_url
	$nhurl = str_ireplace(site_url(), home_url(), $hurl);
	$f = str_ireplace(rtrim($wp_home, '/'), rtrim($wp_home_path, '/'), $nhurl);
	if (file_exists($f)) { 
		if($type == 'js') { $code = fastvelocity_min_get_js($hurl, file_get_contents($f), $disable_minification); } 
		else { $code = fastvelocity_min_get_css($hurl, file_get_contents($f).$inline, $disable_minification); }
		set_transient($tkey, $code, 7 * DAY_IN_SECONDS );
		fvm_update_transient_keys($tkey); # keep track
		$log = " - FILE &nbsp; - $printurl --- Debug: File was opened from $f ---\n";
		return array('log'=>$log, 'code'=>$code);
	}
}
}

# else, fallback to remote urls (or windows)
$response = wp_remote_get($hurl, array('user-agent'=>$uagent, 'timeout' => 10, 'httpversion' => '1.1', 'sslverify'=>false)); 
$response_code = wp_remote_retrieve_response_code($response);

if($response_code == '200') { 			
	$coderesponse = wp_remote_retrieve_body($response);
	if($type == 'js') { $code = fastvelocity_min_get_js($hurl, $coderesponse, $disable_minification); } 
	else { $code = fastvelocity_min_get_css($hurl, $coderesponse.$inline, $disable_minification); }
	set_transient($tkey, $code, 7 * DAY_IN_SECONDS );
	fvm_update_transient_keys($tkey); # keep track
	$log = " - URL &nbsp;&nbsp; - $printurl --- Debug: Fetched via wp_remote_get at $hurl \n";
	return array('log'=>$log, 'code'=>$code);
}

# retry if 40x error, same domain and home_url != site_url 
if($response_code >= '400' && stripos($hurl, $wp_domain) !== false && home_url() != site_url()) {
	$nhurl = str_ireplace(site_url(), home_url(), $hurl);
	$nresponse = wp_remote_get($nhurl, array('user-agent'=>$uagent, 'timeout' => 10, 'httpversion' => '1.1', 'sslverify'=>false));
	$nresponse_code = wp_remote_retrieve_response_code($nresponse); # Open as Safari 8
	if($response_code == '200') { 			
		$coderesponse = wp_remote_retrieve_body($response);
		if($type == 'js') { $code = fastvelocity_min_get_js($hurl, $coderesponse, $disable_minification); } 
		else { $code = fastvelocity_min_get_css($hurl, $coderesponse.$inline, $disable_minification); }
		set_transient($tkey, $code, 7 * DAY_IN_SECONDS );
		fvm_update_transient_keys($tkey); # keep track
		$log = " - URL &nbsp;&nbsp; - $printurl --- Debug: Fetched via wp_remote_get at $hurl ---\n";
		return array('log'=>$log, 'code'=>$code);
	}
}
	
# still no luck, let's try curl if available
if(function_exists('curl_version')) {
	$curl = fvm_file_get_contents_curl($hurl, $uagent);
	if(!empty($curl) && strlen($curl) > 1) {
		$log = " - CURL &nbsp; - $printurl --- Debug: Fetched via curl at $hurl ---\n";
		return array('log'=>$log, 'code'=>$curl);
	}
}


# if remote urls failed... try to open locally again, regardless of OS in use
if (stripos($hurl, $wp_domain) !== false) { 
	# default
	$f = str_ireplace(rtrim($wp_home, '/'), rtrim($wp_home_path, '/'), $hurl);
	if (file_exists($f)) { 
		if($type == 'js') { $code = fastvelocity_min_get_js($hurl, file_get_contents($f), $disable_minification); } 
		else { $code = fastvelocity_min_get_css($hurl, file_get_contents($f).$inline, $disable_minification); }
		set_transient($tkey, $code, 7 * DAY_IN_SECONDS );
		fvm_update_transient_keys($tkey); # keep track
		$log = " - FILE &nbsp; - $printurl --- Debug: File was opened from $f ---\n";
		return array('log'=>$log, 'code'=>$code);
	}
	
	# failover when home_url != site_url
	$nhurl = str_ireplace(site_url(), home_url(), $hurl);
	$f = str_ireplace(rtrim($wp_home, '/'), rtrim($wp_home_path, '/'), $nhurl);
	if (file_exists($f)) { 
		if($type == 'js') { $code = fastvelocity_min_get_js($hurl, file_get_contents($f), $disable_minification); } 
		else { $code = fastvelocity_min_get_css($hurl, file_get_contents($f).$inline, $disable_minification); }
		set_transient($tkey, $code, 7 * DAY_IN_SECONDS );
		fvm_update_transient_keys($tkey); # keep track
		$log = " - FILE &nbsp; - $printurl --- Debug: File was opened from $f ---\n";
		return array('log'=>$log, 'code'=>$code);
	}
}

	
# else fail
$code = false; 
$log = " - FAILED --- Debug: Tried to fetch via wp_remote_get, curl and also to open it locally. URL: $hurl ---\n";
return array('log'=>$log, 'code'=>$code);
}



# Concatenate Google Fonts tags (http://fonts.googleapis.com/css?...)
function fastvelocity_min_concatenate_google_fonts($array) {

# extract unique font families
$families = array(); foreach ($array as $font) {

# get fonts name, type and subset, remove wp query strings
$font = explode('family=', htmlspecialchars_decode(rawurldecode(urldecode($font))));
$a = explode('&v', end($font)); $font = trim(trim(trim(current($a)), ','));

# reprocess if fonts are already concatenated in this url
if(stristr($font, '|') !== FALSE) { 
	$multiple = explode('|', $font); if (count($multiple) > 0) { foreach ($multiple as $f) { $families[] = trim($f); } }
} else { $families[] = $font; }
}

# process types, subsets, merge, etc
$fonts = array(); 
foreach ($families as $font) {
		
# if no type or subset
if(stristr($font, ':') === FALSE) { 
	$fonts[] = array('name'=>$font, 'type'=>'', 'sub'=>''); 
} else {

	# get type and subset
	$name = stristr($font, ':', true);       # font name, before :
	$ftype = trim(stristr($font, ':'), ':'); # second part of the string, after :

	# get font types and subset
	if(stristr($ftype, '&subset=') === FALSE) { 
		$fonts[] = array('name'=>$name, 'type'=>$ftype, 'sub'=>''); 
	} else { 
		$newftype = stristr($ftype, '&', true);        # font type, before &
		$subset = trim(str_ireplace('&subset=', '', stristr($ftype, '&')));     # second part of the string, after &
		$fonts[] = array('name'=>$name, 'type'=>$newftype, 'sub'=>$subset); 
	}

}
}

# make sure we have unique font names, types and subsets
$ufonts = array(); foreach ($fonts as $f) { $ufonts[$f['name']] = $f['name']; }                              # unique font names
$usubsets = array(); foreach ($fonts as $f) { if(!empty($f['sub'])) { $usubsets[$f['sub']] = $f['sub']; } }  # unique subsets

# prepare
$fonts_and_types = $ufonts;

# get unique types and subsets for each unique font name
foreach ($ufonts as $uf) {
	
	# types
	$utypes = array(); 
	foreach ($fonts as $f) {
		if($f['name'] == $uf && !empty($f['type'])) { $utypes = array_merge($utypes, explode(',', $f['type'])); }
	}
	
	# filter types
	$utypes = array_unique($utypes);
    sort($utypes);
	$ntype = ''; if(count($utypes) > 0) { $ntype = ':'.implode(',', $utypes); } # types to append to the font name
	
	# generate font url queries
	$fonts_and_types[$uf] = str_ireplace(' ', '+', $uf).$ntype;
}

# concat fonts, generate unique google fonts url
if(count($fonts_and_types) > 0) {
	$msubsets = ''; if(count($usubsets) > 0 && implode(',', $usubsets) != 'latin') { $msubsets = "&subset=".implode(',', $usubsets); } # merge subsets
	return trim('https://fonts.googleapis.com/css?family='.implode('|', $fonts_and_types).$msubsets); # process
}

return false;
}


# readme parser
function fastvelocity_min_readme($url) {

	# read file
	$file = @file_get_contents( $url );
	if (empty($file)) { return '<strong>Readme Parser: readme.txt not found!</strong>'; }
	
	// line end to \n
	$file = preg_replace("/(\n\r|\r\n|\r|\n)/", "\n", $file);

	// headlines
	$s = array('===','==','=' ); 
	$r = array('h2' ,'h3','h4');
	for ( $x = 0; $x < sizeof($s); $x++ ) { 
		$file = preg_replace('/(.*?)'.$s[$x].'(?!\")(.*?)'.$s[$x].'(.*?)/', '$1<'.$r[$x].'>$2</'.$r[$x].'>$3', $file); 
	}

	// inline
	$s = array('\*\*','\`'); 
	$r = array('b'   ,'code');
	for ( $x = 0; $x < sizeof($s); $x++ ) { 
		$file = preg_replace('/(.*?)'.$s[$x].'(?!\s)(.*?)(?!\s)'.$s[$x].'(.*?)/', '$1<'.$r[$x].'>$2</'.$r[$x].'>$3', $file); 
	}
	
	// ' _italic_ '
	$file = preg_replace('/(\s)_(\S.*?\S)_(\s|$)/', ' <em>$2</em> ', $file);
	
	// ul lists	
	$s = array('\*','\+','\-');
	for ( $x = 0; $x < sizeof($s); $x++ )
	$file = preg_replace('/^['.$s[$x].'](\s)(.*?)(\n|$)/m', '<li>$2</li>', $file);
	$file = preg_replace('/\n<li>(.*?)/', '<ul><li>$1', $file);
	$file = preg_replace('/(<\/li>)(?!<li>)/', '$1</ul>', $file);
	
	// ol lists
	$file = preg_replace('/(\d{1,2}\.)\s(.*?)(\n|$)/', '<li>$2</li>', $file);
	$file = preg_replace('/\n<li>(.*?)/', '<ol><li>$1', $file);
	$file = preg_replace('/(<\/li>)(?!(\<li\>|\<\/ul\>))/', '$1</ol>', $file);
	
	// ol screenshots style
	$file = preg_replace('/(?=Screenshots)(.*?)<ol>/', '$1<ol class="readme-parser-screenshots">', $file);
	
	// line breaks
	$file = preg_replace('/(.*?)(\n)/', "<p>$1</p>", $file);
	$file = preg_replace('/(1|2|3|4)(><br\/>)/', '$1>', $file);
	$file = str_ireplace('</ul><br/>', '</ul>', $file);
	
	# cleanup
	$file = str_ireplace('<p></p>', '', $file);
	$file = str_ireplace('<p><h4>', '<h4>', $file);
	$file = str_ireplace('</h4></p>', '</h4>', $file);
	
	// urls
	$file = str_replace('http://www.', 'www.', $file);
	$file = str_replace('www.', 'http://www.', $file);
	$file = preg_replace('#(^|[^\"=]{1})(http://|ftp://|mailto:|https://)([^\s<>]+)([\s\n<>]|$)#', '$1<a target="_blank" href="$2$3">$2$3</a>$4', $file);
	
	# extract faqs
	$prefix = "Frequently Asked Questions";
	$faq = substr($file, strpos($file, $prefix) + strlen($prefix));
	$faq = substr($faq, 0, strpos($faq, '<p><h3>'));
	
	
	return trim($faq);
}


# remove emoji support
function fastvelocity_min_disable_wp_emojicons() {
remove_action('wp_head', 'print_emoji_detection_script', 7);
remove_action('wp_print_styles', 'print_emoji_styles');
remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
remove_action( 'admin_print_styles', 'print_emoji_styles' );
}


# remove UTF8 BOM
function fastvelocity_min_remove_utf8_bom($text) {
    $bom = pack('H*','EFBBBF');
    $text = preg_replace("/^$bom/ui", '', $text);
    return $text;
}


# Remove query string from static css files
function fastvelocity_remove_cssjs_ver( $src ) {
 if(stripos($src, '?ver=')) { $src = remove_query_arg('ver', $src); }
 return $src;
}


# rewrite cache files to http, https or dynamic
function fvm_get_protocol($cache_url) {
	$cache_url = ltrim(str_ireplace(array('http://', 'https://'), '', $cache_url), '/'); # better compatibility
	$fp = get_option('fastvelocity_min_default_protocol', 'dynamic'); 
	if($fp == 'http') { $cache_url = 'http://'.$cache_url; }
	elseif($fp == 'https') { $cache_url = 'https://'.$cache_url; }
	else { $cache_url = '//'.$cache_url; }
	return $cache_url;
}


# keep track of transients, whenever we save a transient.
function fvm_update_transient_keys($new_transient_key) {
$transient_keys = get_option('fvm_transient_keys'); # get
$transient_keys[]= $new_transient_key; # append
update_option( 'fvm_transient_keys', $transient_keys); # save
}

# keep track of transients, dump our plugin transients as needed
function fvm_transients_purge() {
$transient_keys = get_option( 'fvm_transient_keys' ); # get
update_option( 'fvm_transient_keys', array()); # reset
foreach( $transient_keys as $t ) { delete_transient( $t ); } # delete
} 



# exclude processing from some pages / posts / contents
function fastvelocity_exclude_contents() {
	
# customizer preview, visual composer
$arr = array('customize_theme', 'preview_id', 'preview');
foreach ($arr as $a) { if(isset($_GET[$a])) { return true; } }

# Thrive plugins and other post_types
$arr = array('tve_form_type', 'tve_lead_shortcode', 'tqb_splash');
foreach ($arr as $a) { if(isset($_GET['post_type']) && $_GET['post_type'] == $a) { return true; } }

# default
return false;
}

# IE only files that should always be ignored
function fastvelocity_default_ignore($ignore) {
if(is_array($ignore)) {
$exc = array('/fvm/cache/', '/html5shiv.js', '/excanvas.js', '/avada-ie9.js', '/respond.js', '/respond.min.js', '/selectivizr.js', '/Avada/assets/js/main.min.js', '/Avada/assets/css/ie.css', '/html5.js', '/IE9.js', '/fusion-ie9.js', '/vc_lte_ie9.min.css', '/old-ie.css', '/ie.css', '/vc-ie8.min.css', '/woocommerce-product-search/js/product-search.js', '/Divi/includes/builder/scripts/frontend-builder-scripts.js');

# should we exclude jquery when defer is enabled?
$exclude_defer_jquery = get_option('fastvelocity_min_exclude_defer_jquery');
$enable_defer_js = get_option('fastvelocity_min_enable_defer_js');
if($enable_defer_js == true && $exclude_defer_jquery == true) {
	$exc[] = '/jquery.js';
	$exc[] = '/jquery.min.js';
}

# make sure it's unique and not empty
$uniq = array();
foreach ($ignore as $i) { $k = hash('adler32', $i); if(!empty($i)) { $uniq[$k] = $i; } }
foreach ($exc as $e) { $k = hash('adler32', $e); if(!empty($e)) { $uniq[$k] = $e; } }

# merge
return $uniq;
} else { return $ignore; }
}



function fastvelocity_cache_purger(){
# Purge all W3 Total Cache
if (function_exists('w3tc_pgcache_flush')) {
do_action( 'w3tc_pgcache_flush' );
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>W3 Total Cache</strong> have also been purged.</p></div>');
}

# Purge WP Super Cache
if (function_exists('wp_cache_clear_cache')) {
do_action( 'wp_cache_clear_cache' );
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>WP Super Cache</strong> have also been purged.</p></div>');
}

# Purge WP Rocket
if (function_exists('rocket_clean_domain')) {
do_action( 'rocket_clean_domain' );
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>WP Rocket</strong> have also been purged.</p></div>');
}

# Purge Wp Fastest Cache
if(isset($GLOBALS['wp_fastest_cache']) && method_exists($GLOBALS['wp_fastest_cache'], 'deleteCache')){
$GLOBALS['wp_fastest_cache']->deleteCache();
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>Wp Fastest Cache</strong> have also been purged.</p></div>');
}

# Purge Cachify
if (function_exists('cachify_flush_cache')) {
do_action( 'cachify_flush_cache' );
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>Cachify</strong> have also been purged.</p></div>');
}

# Purge Comet Cache
if ( class_exists("comet_cache") ) {
comet_cache::clear();
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>Comet Cache</strong> have also been purged.</p></div>');
}

# Purge Zen Cache
if ( class_exists("zencache") ) {
zencache::clear();
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>Comet Cache</strong> have also been purged.</p></div>');
}

# Purge SG Optimizer
if (function_exists('sg_cachepress_purge_cache')) {
sg_cachepress_purge_cache();
return __('<div class="notice notice-info is-dismissible"><p>All caches from <strong>SG Optimizer</strong> have also been purged.</p></div>');
}


}
